@@ -12,12 +12,11 @@ from .child import Child
 from .cli import parse_args
 from .cli_stub import LaunchCLIOptions
 from .clipboard import set_clipboard_string, set_primary_selection
-from .constants import shell_path
 from .fast_data_types import add_timer, get_boss, get_options, get_os_window_title, patch_color_profiles
 from .options.utils import env as parse_env
 from .tabs import Tab, TabManager
 from .types import OverlayType, run_once
-from .utils import cmdline_for_hold, get_editor, log_error, resolve_custom_file, which
+from .utils import get_editor, log_error, resolve_custom_file, which
 from .window import CwdRequest, CwdRequestType, Watchers, Window
 
 try:
@@ -72,7 +71,11 @@ Where to launch the child process:
     intended to run for a long time as a primary window.
 
 :code:`background`
-    The process will be run in the :italic:`background`, without a kitty window.
+    The process will be run in the :italic:`background`, without a kitty
+    window. Note that if :option:`kitten @ launch --allow-remote-control` is
+    specified the :envvar:`KITTY_LISTEN_ON` environment variable will be set to
+    a dedicated socket pair file descriptor that the process can use for remote
+    control.
 
 :code:`clipboard`, :code:`primary`
     These two are meant to work with :option:`--stdin-source <launch --stdin-source>` to copy
@@ -137,8 +140,7 @@ Copy the environment variables from the currently active window into the newly
 launched child process. Note that this only copies the environment when the
 window was first created, as it is not possible to get updated environment variables
 from arbitrary processes. To copy that environment, use either the :ref:`clone-in-kitty
-<clone_shell>` feature or the kitty remote control feature with :option:`kitty
-@ launch --copy-env`.
+<clone_shell>` feature or the kitty remote control feature with :option:`kitten @ launch --copy-env`.
 
 
 --location
@@ -323,10 +325,12 @@ def parse_launch_args(args: Optional[Sequence[str]] = None) -> LaunchSpec:
     return LaunchSpec(opts, args)
 
 
-def get_env(opts: LaunchCLIOptions, active_child: Optional[Child] = None) -> Dict[str, str]:
+def get_env(opts: LaunchCLIOptions, active_child: Optional[Child] = None, base_env: Optional[Dict[str,str]] = None) -> Dict[str, str]:
     env: Dict[str, str] = {}
     if opts.copy_env and active_child:
         env.update(active_child.foreground_environ)
+    if base_env is not None:
+        env.update(base_env)
     for x in opts.env:
         for k, v in parse_env(x, env):
             env[k] = v
@@ -394,6 +398,15 @@ def load_watch_modules(watchers: Iterable[str]) -> Optional[Watchers]:
         w = m.get('on_focus_change')
         if callable(w):
             ans.on_focus_change.append(w)
+        w = m.get('on_set_user_var')
+        if callable(w):
+            ans.on_set_user_var.append(w)
+        w = m.get('on_title_change')
+        if callable(w):
+            ans.on_title_change.append(w)
+        w = m.get('on_cmd_startstop')
+        if callable(w):
+            ans.on_cmd_startstop.append(w)
     return ans
 
 
@@ -410,6 +423,7 @@ class LaunchKwds(TypedDict):
     cmd: Optional[List[str]]
     overlay_for: Optional[int]
     stdin: Optional[bytes]
+    hold: bool
 
 
 def apply_colors(window: Window, spec: Sequence[str]) -> None:
@@ -457,6 +471,7 @@ def _launch(
     active: Optional[Window] = None,
     is_clone_launch: str = '',
     rc_from_window: Optional[Window] = None,
+    base_env: Optional[Dict[str, str]] = None,
 ) -> Optional[Window]:
     active = active or boss.active_window_for_cwd
     if active:
@@ -471,7 +486,7 @@ def _launch(
     if opts.os_window_title == 'current':
         tm = boss.active_tab_manager
         opts.os_window_title = get_os_window_title(tm.os_window_id) if tm else None
-    env = get_env(opts, active_child)
+    env = get_env(opts, active_child, base_env)
     remote_control_restrictions: Optional[Dict[str, Sequence[str]]] = None
     if opts.allow_remote_control and opts.remote_control_password:
         from kitty.options.utils import remote_control_password
@@ -490,7 +505,8 @@ def _launch(
         'marker': opts.marker or None,
         'cmd': None,
         'overlay_for': None,
-        'stdin': None
+        'stdin': None,
+        'hold': False,
     }
     spacing = {}
     if opts.spacing:
@@ -581,7 +597,10 @@ def _launch(
         cmd = kw['cmd']
         if not cmd:
             raise ValueError('The cmd to run must be specified when running a background process')
-        boss.run_background_process(cmd, cwd=kw['cwd'], cwd_from=kw['cwd_from'], env=env or None, stdin=kw['stdin'])
+        boss.run_background_process(
+            cmd, cwd=kw['cwd'], cwd_from=kw['cwd_from'], env=env or None, stdin=kw['stdin'],
+            allow_remote_control=kw['allow_remote_control'], remote_control_passwords=kw['remote_control_passwords']
+        )
     elif opts.type in ('clipboard', 'primary'):
         stdin = kw.get('stdin')
         if stdin is not None:
@@ -590,11 +609,7 @@ def _launch(
             else:
                 set_primary_selection(stdin)
     else:
-        if opts.hold:
-            cmd = kw['cmd'] or [shell_path]
-            if not os.path.isabs(cmd[0]):
-                cmd[0] = which(cmd[0]) or cmd[0]
-            kw['cmd'] = cmdline_for_hold(cmd)
+        kw['hold'] = opts.hold
         if force_target_tab:
             tab = target_tab
         else:
@@ -634,12 +649,13 @@ def launch(
     active: Optional[Window] = None,
     is_clone_launch: str = '',
     rc_from_window: Optional[Window] = None,
+    base_env: Optional[Dict[str, str]] = None,
 ) -> Optional[Window]:
     active = active or boss.active_window_for_cwd
     if opts.keep_focus and active:
         orig, active.ignore_focus_changes = active.ignore_focus_changes, True
     try:
-        return _launch(boss, opts, args, target_tab, force_target_tab, active, is_clone_launch, rc_from_window)
+        return _launch(boss, opts, args, target_tab, force_target_tab, active, is_clone_launch, rc_from_window, base_env)
     finally:
         if opts.keep_focus and active:
             active.ignore_focus_changes = orig
@@ -907,6 +923,12 @@ def clone_and_launch(msg: str, window: Window) -> None:
             cmdline = []
         if not cmdline:
             cmdline = list(window.child.argv)
+        if cmdline and cmdline[0].startswith('-'):  # on macOS, run via run-shell kitten
+            if window.child.is_default_shell:
+                cmdline = window.child.unmodified_argv
+            else:
+                cmdline[0] = cmdline[0][1:]
+                cmdline[0] = which(cmdline[0]) or cmdline[0]
         if cmdline and cmdline[0] == window.child.final_argv0:
             cmdline[0] = window.child.final_exe
         if cmdline and cmdline == [window.child.final_exe] + window.child.argv[1:]:
