@@ -3,16 +3,25 @@
 
 import base64
 import sys
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Set, Union
+from typing import TYPE_CHECKING, Any, Dict, Optional, Set, Union
 
-from kitty.fast_data_types import KeyEvent as WindowSystemKeyEvent, get_boss
+from kitty.fast_data_types import KeyEvent as WindowSystemKeyEvent
+from kitty.fast_data_types import get_boss
 from kitty.key_encoding import decode_key_event_as_window_system_key
 from kitty.options.utils import parse_send_text_bytes
 
 from .base import (
-    MATCH_TAB_OPTION, MATCH_WINDOW_OPTION, ArgsType, Boss, CmdGenerator,
-    MatchError, PayloadGetType, PayloadType, RCOptions, RemoteCommand,
-    ResponseType, Window
+    MATCH_TAB_OPTION,
+    MATCH_WINDOW_OPTION,
+    ArgsType,
+    Boss,
+    CmdGenerator,
+    PayloadGetType,
+    PayloadType,
+    RCOptions,
+    RemoteCommand,
+    ResponseType,
+    Window,
 )
 
 if TYPE_CHECKING:
@@ -63,19 +72,21 @@ class FocusChangedSession(SessionAction):
 
 
 class SendText(RemoteCommand):
-    '''
-    data+/send_text: The data being sent. Can be either: text: followed by text or base64: followed by standard base64 encoded bytes
+    protocol_spec = __doc__ = '''
+    data+/str: The data being sent. Can be either: text: followed by text or base64: followed by standard base64 encoded bytes
     match/str: A string indicating the window to send text to
     match_tab/str: A string indicating the tab to send text to
     all/bool: A boolean indicating all windows should be matched.
     exclude_active/bool: A boolean that prevents sending text to the active window
-    session_id/internal: A string that identifies a "broadcast session"
+    session_id/str: A string that identifies a "broadcast session"
     '''
     short_desc = 'Send arbitrary text to specified windows'
     desc = (
         'Send arbitrary text to specified windows. The text follows Python'
-        ' escaping rules. So you can use escapes like :code:`\\x1b` to send control codes'
-        ' and :code:`\\u21fa` to send unicode characters. If you use the :option:`kitty @ send-text --match` option'
+        ' escaping rules. So you can use :link:`escapes <https://www.gnu.org/software/bash/manual/html_node/ANSI_002dC-Quoting.html>`'
+        " like :code:`'\\\\e'` to send control codes"
+        " and :code:`'\\\\u21fa'` to send Unicode characters. Remember to use single-quotes otherwise"
+        ' the backslash is interpreted as a shell escape character. If you use the :option:`kitten @ send-text --match` option'
         ' the text will be sent to all matched windows. By default, text is sent to'
         ' only the currently active window.'
     )
@@ -100,8 +111,7 @@ not interpreted for escapes. If stdin is a terminal, you can press :kbd:`Ctrl+D`
 Path to a file whose contents you wish to send. Note that in this case the file contents
 are sent as is, not interpreted for escapes.
 '''
-    no_response = True
-    argspec = '[TEXT TO SEND]'
+    args = RemoteCommand.Args(spec='[TEXT TO SEND]', json_field='data', special_parse='+session_id:parse_send_text(io_data, args)')
 
     def message_to_kitty(self, global_opts: RCOptions, opts: 'CLIOptions', args: ArgsType) -> PayloadType:
         limit = 1024
@@ -134,9 +144,10 @@ are sent as is, not interpreted for escapes.
                     yield ret
 
         def chunks(text: str) -> CmdGenerator:
-            data = parse_send_text_bytes(text).decode('utf-8')
+            data = parse_send_text_bytes(text)
             while data:
-                ret['data'] = f'text:{data[:limit]}'
+                b = base64.standard_b64encode(data[:limit]).decode("ascii")
+                ret['data'] = f'base64:{b}'
                 yield ret
                 data = data[limit:]
 
@@ -166,24 +177,7 @@ are sent as is, not interpreted for escapes.
 
     def response_from_kitty(self, boss: Boss, window: Optional[Window], payload_get: PayloadGetType) -> ResponseType:
         sid = payload_get('session_id', '')
-        if payload_get('all'):
-            windows: List[Optional[Window]] = list(boss.all_windows)
-        else:
-            windows = [boss.active_window]
-            match = payload_get('match')
-            if match:
-                windows = list(boss.match_windows(match))
-                if not windows and not sid:
-                    raise MatchError(payload_get('match'))
-            mt = payload_get('match_tab')
-            if mt:
-                windows = []
-                tabs = tuple(boss.match_tabs(mt))
-                if not tabs and not sid:
-                    raise MatchError(payload_get('match_tab'), 'tabs')
-                for tab in tabs:
-                    if tab:
-                        windows += tuple(tab)
+        windows = self.windows_for_payload(boss, None, payload_get, window_match_name='match')
         pdata: str = payload_get('data')
         encoding, _, q = pdata.partition(':')
         session = ''
